function [errors, diffs, epsilon0s, eff_dim, mod_eff_dim, correct_parameters] = ...
    arx_comparison(numerator, denomenator, input, snr, lambda, delta, resolution, preceding_input)
%ARX_COMPARISON Perform on-line system identification and compare the
%   performance and accuracy of various on-line estimators.
%   [E, DIFF, E0, EFF_DIM, MOD_EFF_DIM, CORRECT] = ...
%       ARX_COMPARISON(NUM, DEN, U, SNR)
%   NUM and DEN are the filter numerator and denomenator, respectively.
%   U is the input. SNR is the signal-to-noise ratio at the filter output.
%   E is a LENGTH(U) by 4 matrix containing the squared errors produced by
%   RLS, SBME, RfrEBME & JS-RLS, respectively, for every moment in time.
%   DIFF is a LENGTH(U) by 3 matrix containing a grade for the accuracy of
%   the recursive calculation of RLS, SBME & RfrEBME. EFF_DIM and
%   MOD_EFF_DIM are the effective dimension and modified effective
%   dimension, respectively, for every moment in time. CORRECT is the
%   true parameter vector.
%
%   [...] = ARX_COMPARISON(NUM, DEN, U, SNR, FF) also specifies the 
%   exponential forgetting factor [1]. If not specified, a forgetting
%   factor of 1 is assumed.
%
%   [...] = ARX_COMPARISON(NUM, DEN, U, SNR, FF, DELTA) also specifies
%   the initialization parameter DELTA. If 0 < DELTA < 1, then a uniform
%   square regularization with parameter DELTA is used. IF DELTA > 1, then
%   the first DELTA*LENGTH(CORRECT) samples are used to initialize the
%   estimator, using INITIAL_ESTIMATION(). If not specified, a value of
%   1e-6 is used.
%
%   [...] = ARX_COMPARISON(NUM, DEN, U, SNR, FF, DELTA, RES) also
%   specifies the resolution at which the MSE of least squares, effective
%   dimension, modified effective dimension and accuracy of the recursive
%   calculation be calculated. This greatly affects running time. If a
%   resolution of 0 is specified, these are never calculated. If not
%   specified at all, a resolution of 1 is assumed.
%
%   [...] = ARX_COMPARISON(NUM, DEN, U, SNR, FF, DELTA, RES, PRE_U) also
%   specifies input preceding that specified in U, which can be used to
%   neutralize any effects of the zero initial condition otherwise assumed.
%
%   ARX_COMPARISON(...) (with no output arguments) plots the results in 
%   various graphs.
%
%   [1] Haykin, Adaptive Filter Theory, 4rd ed., pp. 437.
%
%   See also ARX_COMPARISON_ITERATE, INITIAL_ESTIMATION, GET_HY.

%   Asaf Elron and Guy Leibovitz, 2008.
%   Based on Leibovitz, Elron, Ben-Haim & Eldar, Recursive Blind Minimax
%   Estimation: Improving MSE over Recursive Least Squares", IEEEI 2008.

%% some input control
if (nargin < 5) % was the forgetting factor not specified?
    lambda = 1; % assume 1
end;
if (nargin < 6) % was the initialization not selected?
    delta = 1e-6; % assume some fair value
end;
if (nargin < 7) % was the resolution not specified?
    resolution = 1;
end;
if (nargin < 8) % was a preceding not specified?
    preceding_input = [];
end;

% assert that input, denomenator & numerator are column vectors
is_col = @(X) (length(size(X)) == 2 && size(X, 2) == 1);
if ~(is_col(input) && is_col(denomenator) && is_col(numerator) )
	error('BME:ARX_COMPARISON:COLUMN_VEC', 'DEN, NUM & U need be column vectors.');
end;

% assert that filter is stable
sys = filt(numerator', denomenator');
if (sum(abs(cell2mat(get(zpk(sys), 'p'))) > 1) > 0)
	warning('BME:ARX_COMPARISON:COLUMN_VEC', 'Given filter is unstable.');
end;

% assert that a valid forgetting factor has been set
if (lambda > 1 || lambda < 0)
	error('BME:ARX_COMPARISON:FF', 'Forgetting factor our of bounds.');
end;

%% setup
a0 = denomenator(1); % first coeficient of the denomenator
N = length(input); % length of input

p = length(denomenator); % length of denomenator
q = length(numerator); % length of numerator
s = max(p, q); %#ok<NASGU>
l = p-1 + q; % length of parameter vector

% set true parameter vector
correct_parameters = [numerator ; -denomenator(2:end)];

% calculate model matrix, noisy output and noise variance used to generate it
[H, y, noise_variance] = get_hy(numerator, denomenator, input, snr, preceding_input);

if (resolution ~= 0) % is non-recursive calculation required?
    % set covariance matrix
    Cw = noise_variance*eye(N);
end;

% set indices into cell array to be defined later
RLS = 1;
RSBM = 2;
RfrEBM = 3;
JSRLS = 4;

%% initialize estimators
if (delta > 1)
    % use some initial measurements
    initial_samples = ceil(delta*l);
    
    % use more initial measurements, if that is required for numerical stability
    while (rcond((H(1:initial_samples,:)' * H(1:initial_samples,:))^2) < 1e-15)
        initial_samples = initial_samples + 1;
        
        if (initial_samples == length(input))
        	error('BME:ARX_COMPARISON:ILL_POSED', 'There are not enough input samples for accurate calculation.');
        end;
    end;
    
    % calculate initial estimation
    delta = initial_estimation(H(1:initial_samples,:), a0*y(1:initial_samples), lambda);
else
    % use no initial measurements
    initial_samples = 0;
end;

% actually initialize estimators
data = { rls_init(l, delta, lambda, noise_variance), ...
         rsbme_init(l, delta, lambda, noise_variance), ...
         rfrebme_init(l, delta, lambda, noise_variance), ...
         jsrls_init(l, delta, lambda, noise_variance) };

%% run through samples
calc_errors = @(data) norm_squared(data.estimation - correct_parameters); % function calculates errors
errors = zeros(N, 4); % errors of estimators for every moment
diffs = NaN*ones(N, 4); % grade for the inaccuracy of recursive calculation for every moment
epsilon0s = zeros(N, 1); % MSE of least squares for every moment
eff_dim = NaN*ones(N, 1); % effective dimension for every moment
mod_eff_dim = NaN*ones(N, 1); % modified effective dimension for every moment
for i = initial_samples+1:N % iterate through all (remaining) measurements
    
    % update the on-line estimators according to the new measurements
    data = { rls(data{RLS}, H(i,:)', a0*y(i)), ...
             rsbme(data{RSBM}, H(i,:)', a0*y(i)), ...
             rfrebme(data{RfrEBM}, H(i,:)', a0*y(i)), ...
             jsrls(data{JSRLS}, H(i,:)', a0*y(i)) };

	% calculate errors
    errors(i,:) = cellfun(calc_errors, data);
    
    % calculate non-recursive estimators
    if (i >= ceil(1.5*l) && mod(i, resolution) == 0) % is it time to calculate non-recursive estimators?
        % LS calculation
        ls = leastsq_prep(H(1:i,:), forget(i, lambda^-1)*Cw(1:i,1:i));
        x_hat_ls = leastsq(ls, a0*y(1:i));
        
        % SBME calculation
        [sbm, eff_dim(i)] = sbme_prep(H(1:i,:), forget(i, lambda^-1)*Cw(1:i,1:i));
        x_hat_sbm = sbme(sbm, a0*y(1:i));
        
        % frEBME calculation
        b = -2; % NOTICE THIS!
        [frebm, mod_eff_dim(i)] = ebme_prep(H(1:i,:), forget(i, lambda^-1)*Cw(1:i,1:i), b);
        x_hat_frebm = frebme(frebm, a0*y(1:i));
        x_hat_ebm = ebme(frebm, a0*y(1:i));

%       [ebm, mod_eff_dim(i)] = ebme_prep(H(1:i,:), Cw(1:i,1:i), -1);
%       x_hat_ebm = ebme(ebm, a0*y(1:i));
        
        % save MSE of least squares
        epsilon0s(i) = sbm.epsilon0;
        
        % calculate the grade for the inaccuracy of recursive calculation 
        diffs(i, 1) = norm(data{RLS}.estimation - x_hat_ls)/norm(x_hat_ls);
        diffs(i, 2) = norm(data{RSBM}.estimation - x_hat_sbm)/norm(x_hat_sbm);
        diffs(i, 3) = norm(data{RfrEBM}.estimation - x_hat_frebm)/norm(x_hat_frebm);
    end;
end;

%% do not plot results if return values are requested
if (nargout > 0)
    return;
end;

%% plot results
figure;
zgrid;
pzmap(filt(numerator', denomenator'));
title('Zero-Pole Map of Filter');

figure;
subplot(3, 1, 1);
semilogy([errors epsilon0s]./norm_squared(correct_parameters));
title('Squared Errors Normalized Relative to ||x||^2');
legend('RLS', 'RSBME', 'RfrEBME', 'JS-RLS', 'epsilon0');
xlabel('Samples');
xlim([1 length(epsilon0s)]);

subplot(3, 1, 2);
ratio = errors./(errors(:,RLS)*ones(1, size(errors, 2)));
plot(ratio);
title('Squared Errors Normalized Relative to LS''s Squared Error');
legend('RLS', 'RSBME', 'RfrEBME', 'JS-RLS');
xlabel('Samples');
xlim([1 length(epsilon0s)]);

subplot(3, 1, 3);
plot([4*ones(N, 1) eff_dim mod_eff_dim]);
title('Effective Dimensions');
legend('4', 'Effective Dimension', 'Modified Effective Dimension');
xlabel('Samples');
xlim([1 length(epsilon0s)]);

if (resolution ~= 0) % was non-recursive calculation required?
    figure;
    plot([x_hat_ls x_hat_sbm x_hat_frebm correct_parameters]);
    title('Non-recursive Estimations and Original');
    legend('LS estimate', 'SBM estimate', 'frEBM estimate', 'original');
    xlim([0 length(correct_parameters)+1]);

    figure;
    plot([x_hat_ls x_hat_sbm x_hat_ebm correct_parameters]);
    title('Non-recursive Estimations and Original (proper EBME)');
    legend('LS estimate', 'SBM estimate', 'EBM estimate', 'original');
    xlim([0 length(correct_parameters)+1]);
end;

figure;
plot([data{RLS}.estimation data{RSBM}.estimation data{RfrEBM}.estimation correct_parameters]);
title('Recursive Estimations and Original');
legend('RLS estimate', 'RSBM estimate', 'RfrEBM estimate', 'original');
xlim([0 length(correct_parameters)+1]);

if (resolution ~= 0) % was non-recursive calculation required?
    figure;
    plot(diffs, 'o');
    title('Recursive Calculation Accuracy');
    legend('LS-RLS diff', 'SBM-RSBM diff', 'frEBME-RfrEBM diff');
    xlabel('Samples');
end;

figure;
plot([input y]);
legend('system input', 'noisy system output');
xlabel('Samples');

end
