function [MSEs, average_epsilon0s, average_eff_dim, average_mod_eff_dim, average_diffs] = ...
    arx_comparison_iterate(numerator, denomenator, input, iterations, snr, ...
    varargin ... % forgetting_factor, delta, resolution, preceding_input
    )
%ARX_COMPARISON_ITERATE Perform repetitions, with different noise
%   realizations, of on-line system identification and compare the
%   performance and accuracy of various on-line estimators.
%   ARX_COMPARISON_ITERATE(NUM, DEN, ITERATIONS, U, SNR) where NUM and
%   DEN are the filter numerator and denominator, respectively. U is the
%   input. The number of iterations is ITERATIONS. SNR is the
%   signal-to-noise ratio at the filter output. Function plots the results
%   in various graphs.
%
%   ARX_COMPARISON_ITERATE(NUM, DEN, U, SNR, FF) also specifies the
%   exponential forgetting factor. If not specified, a forgetting factor of
%   1 is assumed.
%
%   ARX_COMPARISON_ITERATE(NUM, DEN, U, SNR, FF, DELTA) also specifies
%   the initialization parameter DELTA. If 0 < DELTA < 1, then a uniform
%   square regularization with parameter DELTA is used. If DELTA > 1, then
%   the first DELTA*LENGTH(CORRECT) samples are used to initialize the
%   estimator, using INITIAL_ESTIMATION(). If not specified, a value of
%   1e-6 is used.
%
%   ARX_COMPARISON_ITERATE(NUM, DEN, U, SNR, FF, DELTA, RES) also
%   specifies the resolution at which the MSE of least squares, effective
%   dimension, modified effective dimension and accuracy of the recursive
%   calculation be calculated. This greatly affects running time. If not
%   specified, a resolution of 1 is assumed.
%
%   ARX_COMPARISON_ITERATE(NUM, DEN, U, SNR, FF, DELTA, RES, PRE_U) also
%   specifies input preceding that specified in U, which can be used to
%   neutralize any effects of the zero initial condition otherwise assumed.
%
%   [1] Haykin, Adaptive Filter Theory, 4rd ed., pp. 437.
%
%   See also ARX_COMPARISON, INITIAL_ESTIMATION, GET_HY.

%   Asaf Elron and Guy Leibovitz, 2008.
%   Based on Leibovitz, Elron, Ben-Haim & Eldar, Recursive Blind Minimax
%   Estimation: Improving MSE over Recursive Least Squares", IEEEI 2008.

%% setup
ebme_eff_dim_warn_state = warning('off', 'BME:EBME_PREP:EFF_DIM');
sbme_eff_dim_warn_state = warning('off', 'BME:SBME_PREP:EFF_DIM');

%% iterate and collect results
[MSEs, average_diffs, average_epsilon0s, average_eff_dim, average_mod_eff_dim, correct_parameters] = ...
    arx_comparison(numerator, denomenator, input, snr, varargin{:});
MSEs = (1/iterations)*MSEs;
average_diffs = (1/iterations)*average_diffs;
average_epsilon0s = (1/iterations)*average_epsilon0s;
average_eff_dim = (1/iterations)*average_eff_dim;
average_mod_eff_dim = (1/iterations)*average_mod_eff_dim;
for i = 2:iterations
    tic
    
    [errors, diffs, epsilon0s, eff_dim, mod_eff_dim] = ...
        arx_comparison(numerator, denomenator, input, snr, varargin{:});
    MSEs = MSEs + (1/iterations)*errors;
    average_diffs = average_diffs + (1/iterations)*diffs;
    average_epsilon0s = average_epsilon0s + (1/iterations)*epsilon0s;
    average_eff_dim = average_eff_dim + (1/iterations)*eff_dim;
    average_mod_eff_dim = average_mod_eff_dim + (1/iterations)*mod_eff_dim;
    
    disp(sprintf(['iteration ' int2str(i) ' took ' num2str(toc) ' seconds']));
end;

%% restore previous global state
warning(ebme_eff_dim_warn_state);
warning(sbme_eff_dim_warn_state);

%% plot stuff
figure;
subplot(3, 1, 1);
semilogy([MSEs average_epsilon0s]./norm_squared(correct_parameters));
title('Ensamble-Average Squared Errors Normalized Relative to ||x||^2');
legend('RLS', 'RSBME', 'RfrEBME', 'JS-RLS', 'epsilon0');
xlabel('Samples');
xlim([1 length(average_epsilon0s)]);

subplot(3, 1, 2);
plot(MSEs./(MSEs(:,1)*[1 1 1 1]));
title('Ensamble-Average Squared Errors Normalized Relative to LS''s Ensamble-Average Squared Error');
legend('RLS', 'RSBME', 'RfrEBME', 'JS-RLS');
xlabel('Samples');
xlim([1 length(average_epsilon0s)]);

subplot(3, 1, 3);
plot([4*ones(length(average_eff_dim), 1) average_eff_dim average_mod_eff_dim]);
title('Effective Dimensions');
legend('4', 'Effective Dimension', 'Modified Effective Dimension');
xlabel('Samples');
xlim([1 length(average_epsilon0s)]);

figure;
plot(average_diffs, 'o');
title('Average Recursive Calculation Accuracy');
legend('LS-RLS diff', 'SBM-RSBM diff', 'frEBME-RfrEBM diff');
xlabel('Samples');
xlim([1 length(average_epsilon0s)]);

figure;
zgrid;
pzmap(filt(numerator', denomenator'));
title('Zero-Pole Map of Filter');

end
