function [x_hat_ebm, k] = ebme(ebme_data, y)
%EBME Ellipsoidal blind minimax estimator.
%   X_HAT_EBM = EBME(PREP, Y) produces the ellipsoidal blind minimax
%   estimatation of X, solving the system Y = H*X + W, where W is colored
%   Gaussian noise.
%   Calculation is based in part on precalculated data PREP.
%
%   [X_HAT_EBM, K] = EBME(PREP, Y) also returns the number of measurements
%   ignored K.
%
%   See also EBME_PREP, SBME, SBME_PREP.

%   Asaf Elron and Guy Leibovitz, 2008.
%   Based on Ben-Haim & Eldar, Blind Minimax Estimation", IEEE TIT, 2007.

%% calculate x_hat_ls and its norm
x_hat_ls = leastsq(ebme_data.LS, y); % calculate the least squares estimation
Q_norm_squared = x_hat_ls' * ebme_data.Q_to_power_b * x_hat_ls; % calculate the squared Q^b-norm of the least squares estimation

%% handle trivial estimation possibility
if (Q_norm_squared == 0) % is the least squares estimation the trivial estimation?
    x_hat_ebm = 0; % return trivial estimator
    k = 0; % return something as k
    return; % we're done now
end;

%% find number of measurements to ignore k
m = length(ebme_data.sigmas_to_power_half_b);
for k = 0:m-1 % iterate through all possible values of K
    alpha = ebme_data.r1_per_k(k+1) / (Q_norm_squared + ebme_data.r2_per_k(k+1)); % calculate alpha (20)
    if (alpha * ebme_data.sigmas_to_power_half_b(k+1) < 1) % (21)
        break;
    end;
end;

%% calculate EBME estimation
x_hat_ebm = ebme_data.V * diag( max(0, 1 - alpha * ebme_data.sigmas_to_power_half_b) ) * ebme_data.V' * x_hat_ls; % (19)

end
