function [ebme_data, modified_effective_dimension] = ebme_prep(H, Cw, b)
%EBME_PREP Prepare ellipsoidal blind minimax estimator.
%   EBME = EBME_PREP(H, Cw, B) is a struct containing preperations for
%   the calculation of the ellipsoidal blind minimax estimation with a 
%   discrimination factor B, to be used with EBME().
%   The measurement transform H is assumed to be of full rank and the noise
%   covariance Cw is assumed to be positive definite.
%
%   EBME = EBME_PREP(H, Cw) assumes a value of -1 for the discrimination
%   factor B, as in the simulations performed in the original paper.
%
%   [EBME, MOD_EFF_DIM] = EBME_PREP(...) also specifies the modified
%   effective dimension.
%
%   A warning is issued if the problem parameters do not guarantee
%   dominance over least squares.
%
%   See also EBME, SBME_PREP, SBME.

%   Asaf Elron and Guy Leibovitz, 2008.
%   Based on Ben-Haim & Eldar, Blind Minimax Estimation", IEEE TIT, 2007.

%% set default value for b
if (nargin < 3)
    b = -1;
end;

%% calculate LS related matrices
LS = leastsq_prep(H, Cw); % calculate LS estimator matrix

Qb = (H' * Cw ^-1 * H) ^ b; % calculate Q^b

%% generate r1_per_k & r2_per_k
% r1_per_k & r2_per_k are two vectors containing the different values of r1
% and r2 for each possible value of k.
[V, Sigma_to_power_b] = eig(Qb); % calculate eigenvalue decomposition of Qb

% sort eigenvalues (to the power b) and corresponding eigenvectors in descending order
[sorted_sigmas_to_the_power_b, sort_indexes] = sort(diag(Sigma_to_power_b), 'descend');
sorted_V = V(:, sort_indexes);

% take sorted eigenvalues to various powers
sorted_sigmas_to_power_half_b = sorted_sigmas_to_the_power_b.^0.5;
sorted_sigmas_to_power_half_b_minus_one = sorted_sigmas_to_the_power_b.^((b/2-1)/b);
sorted_sigmas_to_power_b_minus_one = sorted_sigmas_to_power_half_b_minus_one.*sorted_sigmas_to_power_half_b;

% calculate r1 and r2 (20) for all possible values of k (21)
r1_per_k = flipud(cumsum(flipud(sorted_sigmas_to_power_half_b_minus_one)));
r2_per_k = flipud(cumsum(flipud(sorted_sigmas_to_power_b_minus_one)));

%% build struct to return
ebme_data = struct('LS', LS, ...
                   'Q_to_power_b', Qb, ...
                   'b', b, ...
                   'V', sorted_V, ...
                   'sigmas_to_power_half_b', sorted_sigmas_to_power_half_b, ...
                   'r1_per_k', r1_per_k, ...
                   'r2_per_k', r2_per_k);

%% check and report on dominance over least squares
lambda0 = sum(sorted_sigmas_to_power_half_b_minus_one); % sum of eigenvalues of Q^(b/2-1)
lambda_max = max(sorted_sigmas_to_power_half_b_minus_one); % largest eigenvalue of Q^(b/2-1)
modified_effective_dimension = lambda0 / lambda_max;
if ~(modified_effective_dimension > 4) % condition for lack of dominance
    warning('BME:EBME_PREP:EFF_DIM', ...
            'Modified effective dimension is %g. This does not guarantee dominance over LS.', ...
            modified_effective_dimension);
end;

end
