function rfrebme_data = rfrebme_init(parameter_count, delta, lambda, noise_variance)
%RFREBME_INIT Initialize recursive full-rank ellipsoidal blind minimax estimator.
%   RFREBME_DATA = RFREBME_INIT(PARAMETER_COUNT, DELTA, LAMBDA, NOISE_VAR)
%   is a struct containing preperatory data for recursively calculating the
%   full-rank ellipsoidal blind minimax estimatator, to be used with
%   RSBME().
%   PARAMETER_COUNT is the number of paramters to estimate. DELTA is either
%   the regularization initialization parameter [1] or initial estimation
%   data created by INITIAL_ESTIMATION(). LAMBDA is the forgetting factor.
%   NOISE_VAR is the variance of the additive noise.
%
%   [1] Haykin, Adaptive Filter Theory, 4th ed., pp. 437.
%
%   See also RFREBME, RSBME_INIT, RSBME.

%   Asaf Elron and Guy Leibovitz, 2008.
%   Based on Leibovitz, Elron, Ben-Haim & Eldar, Recursive Blind Minimax
%   Estimation: Improving MSE over Recursive Least Squares", IEEEI 2008.

%% 

% initialize RLS
rls_data = rls_init(parameter_count, delta, lambda, noise_variance);

% get local copies, for simpler code
P = rls_data.P; % P = noise_variance^-1 * Q^-1
P_to_power_2 = rls_data.P^2;
noise_variance = rls_data.noise_variance;

% calculate actual estimation x_hat_rfrebm
r1 = trace(P_to_power_2) * noise_variance^2;
r2 = sum(sum(P_to_power_2'.*P))*noise_variance^3;
x_hat_ls_squared_Ptob_norm = rls_data.estimation'*P_to_power_2*rls_data.estimation * noise_variance^2;

alpha = r1 / (x_hat_ls_squared_Ptob_norm + r2);

x_hat_rebm = (eye(size(P,1)) - alpha*P * noise_variance) * rls_data.estimation;

% generate tracking struct
rfrebme_data = struct('rls_data', rls_data, ...
                      'P_to_power_2', P_to_power_2, ...
                      'estimation', x_hat_rebm);

end
