function rsbme_data = rsbme(old_rsbme_data, model_ext_row, measure_ext)
%RSBME Recursive spherical blind minimax estimator.
%   RSBME_DATA = RSBME(OLD_RSBME_DATA, H_EXT, Y_EXT) is a struct containing
%   the data required to recursively calculate the spherical blind minimax
%   estimation. RSBME_DATA.estimation is the current estimate of x, which
%   is updated from OLD_RSBME_DATA using H_EXT and Y_EXT, the extensions
%   to the model matrix and measurements vector respectively.
%
%   In other words, updated is the spherical blind minimax estimate of the
%   vector X in the problem Y_n = H_n * X + W_n , where Y_n is the
%   measurements vector, H_n is the model matrix and W_n is white Gaussian
%   noise. Y_EXT is the measurement added to the bottom of Y_{n-1} to
%   create Y_n, while H_EXT is the transpose of the row added to the bottom
%   of H_{n-1} to create H_n.
%
%   See also RSBME_INIT, RFREBME, RFREBME_INIT.

%   Asaf Elron and Guy Leibovitz, 2008.
%   Based on Leibovitz, Elron, Ben-Haim & Eldar, Recursive Blind Minimax
%   Estimation: Improving MSE over Recursive Least Squares", IEEEI 2008.

%% calculate stuff
rls_data = rls(old_rsbme_data.rls_data, model_ext_row, measure_ext);
x_hat_ls = rls_data.estimation; % get LS estimation on measurements

epsilon0 = trace(rls_data.P) * rls_data.noise_variance; % MSE of LS (3.5)

L_square = norm_squared(x_hat_ls); % calculate x_hat_ls's squared norm

x_hat_sbm = L_square / (L_square + epsilon0) * x_hat_ls; % calculate SBM estimation (3.7)

% save updated tracking struct
rsbme_data = struct('rls_data', rls_data, ...
                    'estimation', x_hat_sbm);

end
