function jsrls_data = jsrls(old_jsrls_data, model_ext_row, measure_ext)
%JSRLS James-Stein Recursive Least Squares estimator.
%   JSRLS_DATA = JSRLS(OLD_JSRLS_DATA, H_EXT, Y_EXT) is a struct containing
%   the data required to recursively calculate the James-Stein recursive
%   least squares estimator. The estimation, in the field 'estimation', is
%   updated from OLD_JSRLS_DATA using H_EXT and Y_EXT, the extensions to
%   the model matrix and measurements vector respectively.
%
%   NOTE: This implementation does not operate in O(m^2) running time.
%
%   See also JSRLS_INIT.

%   Asaf Elron and Guy Leibovitz, 2008.
%   Based on Manton et al., JamesStein State Filtering Algorithms", IEEE
%   SP, 1998.

%% calculate stuff
rls_data = rls(old_jsrls_data.rls_data, model_ext_row, measure_ext);
x_hat_ls = rls_data.estimation; % get LS estimation on measurements
P = rls_data.P; % proportional to LS's covariance matrix
noise_variance = rls_data.noise_variance; % noise variance

lammax = max(eig(P)); % maximal eigenvalue of P
p_star = trace(P)/lammax; % effective dimension
p = length(x_hat_ls); % dimension

squared_Q_norm = x_hat_ls'*P^-1*x_hat_ls; % calculate x_hat_ls's squared Q-norm

% calculate the estimator
max0 = @(x) max(x,0);
x_hat_js = max0(1 - noise_variance * max0(min(p-2,2*(p_star-2)))/squared_Q_norm) * x_hat_ls;

% save updated tracking struct
jsrls_data = struct('rls_data', rls_data, ...
                    'estimation', x_hat_js);

end
