function rls_data = rls(old_rls_data, model_ext_row, measure_ext)
%RLS Recursive least squares estimator.
%   RLS_DATA = RLS(OLD_RLS_DATA, H_EXT, Y_EXT) is a struct containing
%   the data required to recursively calculate the least squares
%   estimation. The estimation, in the field 'estimation', is updated
%   from OLD_RLS_DATA using H_EXT and Y_EXT, the extensions to the model
%   matrix and measurements vector respectively.
%
%   In other words, updated is the least squares estimate of the vector X
%   in the problem Y_n = H_n * X + W_n , where Y_n is the measurements
%   vector, H_n is the model matrix and W_n is white Gaussian noise. Y_EXT
%   is the measurement added to the bottom of Y_{n-1} to create Y_n, while
%   H_EXT is the transpose of the row added to the bottom of H_{n-1} to
%   create H_n.
%
%   See also RLS_INIT, INITIAL_ESTIMATION.

%   Asaf Elron and Guy Leibovitz, 2008.

%% calculate stuff

% get local copies, for simpler code
lambda = old_rls_data.forgetting_factor;
old_P = old_rls_data.P;
old_w = old_rls_data.estimation;

% update
phi = old_P*model_ext_row;
phi_again = model_ext_row'*old_P;
gamma = 1/(lambda + model_ext_row'*phi);
k = gamma*phi;
y = model_ext_row'*old_w;
e = measure_ext - y;
w = old_w + k*e; % update estimation
P = ( old_P - k*phi_again ) / lambda;

% save updated tracking struct
rls_data = struct('noise_variance', old_rls_data.noise_variance, ...
                  'estimation', w, ...
                  'P', P, ...
                  'phi', phi, ...
                  'phi_again', phi_again, ...
                  'gamma', gamma, ...
                  'forgetting_factor', lambda);

end
