function rls_data = rls_init(parameter_count, delta_or_initial_data, lambda, noise_variance)
%RLS_INIT Initialize recursive least squares estimator.
%   RLS_DATA = RLS_INIT(PARAMETER_COUNT, DELTA, LAMBDA, NOISE_VAR) is a
%   struct containing preperatory data for recursively calculating the 
%   least squares estimator, to be used with RLS().
%   PARAMETER_COUNT is the number of paramters to estimate. DELTA is either
%   the regularization initialization parameter [1] or initial estimation
%   data created by INITIAL_ESTIMATION(). LAMBDA is the forgetting factor.
%   NOISE_VAR is the variance of the additive noise.
%
%   [1] Haykin, Adaptive Filter Theory, 4rd ed., pp. 437.
%
%   See also RLS, INITIAL_ESTIMATION.

%   Asaf Elron and Guy Leibovitz, 2008.

% handle both types of initialization
if (isstruct(delta_or_initial_data))
    % tracking initialized with initial estimation
    P = delta_or_initial_data.Qinv;
    estimation = delta_or_initial_data.initial_estimation;
else
    % tracking initialized with regularization factor
    P = delta_or_initial_data^-1*eye(parameter_count);
    estimation = zeros(parameter_count, 1);
end;

% generate tracking struct
rls_data = struct('noise_variance', noise_variance, ...
                  'estimation', estimation, ...
                  'P', P, ...
                  'phi', NaN*ones(parameter_count, 1), ...
                  'phi_again', NaN*ones(parameter_count, 1), ...
                  'gamma', NaN, ...
                  'forgetting_factor', lambda);

end
